@@ -139,8 +139,13 @@ class SwapDecoder:
             result["affiliate"] = parts[4]
 
         # Parse affiliate fee
-        if len(parts) > 5 and parts[5]:
-            result["affiliate_fee"] = int(parts[5])
+        try:
+            if len(parts) > 5 and parts[5]:
+                result["affiliate_fee"] = int(parts[5])
+        except ValueError:
+            # some txs have non-integer affiliate fees
+            # e.g. 0x6264da18732889def93336b2bd6f14d70471d9e7bb626d7012f683d6061e3480_I460
+            result["affiliate_fee"] = None
 
         return result
 
@@ -292,12 +297,20 @@ def get_bridges_from_thorchain_send(
         to_network_thor = asset_code.split(".")[0]
         # asset = asset_code.split(".")[1]
         to_network = THOR_TO_GRAPHSENSE_NETWORK.get(to_network_thor)
+
+        # Check if the target network is supported
+        if to_network is None:
+            logger.warning(
+                f"Skipping thorchain transaction: unsupported network '{to_network_thor}' in memo: {memo}"
+            )
+            return
+
         # to_asset = "native" if asset == network else asset
         # to_amount = swap_info["limit"]  # should be determined from the linked tx
 
     else:
         logger.warning(f"Skipping thorchain action, not implemented memo: {memo}")
-        return None
+        return
 
     from_network = network
 
