@@ -377,7 +377,8 @@ def visualize_graph(
     plt.legend(handles=legend_elements, loc="upper right", bbox_to_anchor=(0.1, 0.1))
 
     # add enough padding
-    plt.show()
+    plt.savefig(f"graph_{tx_hash}.png")
+    plt.close()
 
 
 def handle_general_swap(
@@ -435,8 +436,6 @@ def handle_general_swap(
     tx_sender = None
     if traces:
         trace0 = traces[0]
-        if trace0.trace_address is None:
-            return None
         if trace0.trace_address == "":
             tx_sender = trace0.from_address
 
@@ -447,7 +446,10 @@ def handle_general_swap(
     # Build graph
     G = nx.DiGraph()
     G.add_edges_from(all_from_tos)
-    assert nx.is_weakly_connected(G), "Graph is not weakly connected"
+    if not nx.is_weakly_connected(G):
+        logger.warning(f"Graph is not weakly connected, stop swap detection, {tx_hash}")
+        return None
+    # assert nx.is_weakly_connected(G), "Graph is not weakly connected"
 
     if visualize:
         visualize_graph(G, tx_hash, swap_from_tos, transfer_edges, tx_sender)
