@@ -19,6 +19,8 @@ from graphsenselib.schema.resources.parquet.account import (
 from graphsenselib.schema.resources.parquet.account_trx import (
     BINARY_COL_CONVERSION_MAP_ACCOUNT_TRX,
 )
+from typing import Optional
+from datetime import datetime
 
 DEFAULT_KEY_ENCODERS = {
     bytes: lambda x: x.hex(),
@@ -74,6 +76,29 @@ class DeltaTableConnector:
         files = [f"{table_path}/{file}" for file in files]
         return files
 
+    def get_last_completed_vacuum_date(self, table: str) -> Optional[datetime]:
+        storage_options = self.get_storage_options()
+        table_path = self.get_table_path(table)
+        # Load the Delta table with storage options
+        dt = deltalake.DeltaTable(table_path, storage_options=storage_options)
+
+        # Get the table history
+        history = dt.history()
+
+        # Filter for VACUUM operations
+        vacuum_operations = sorted(
+            [h for h in history if h["operation"] == "VACUUM END"],
+            key=lambda x: x["timestamp"],
+            reverse=True,
+        )
+
+        if len(vacuum_operations) > 0:
+            # Get the most recent vacuum operation
+            last_vacuum = vacuum_operations[0]  # History is sorted by timestamp desc
+            return datetime.fromtimestamp(last_vacuum["timestamp"] // 1000)
+        else:
+            return None
+
     def list_partitions(self, table: str) -> List[int]:
         table_path = self.get_table_path(table)
         storage_options = self.get_storage_options()
@@ -92,6 +117,7 @@ class DeltaTableConnector:
             password = self.s3_credentials.get("AWS_SECRET_ACCESS_KEY")
             access_key = self.s3_credentials.get("AWS_ACCESS_KEY_ID")
 
+            # Enhanced HTTP settings for reliability
             auth_query = f"""
             INSTALL httpfs;
             LOAD httpfs;
@@ -101,9 +127,21 @@ class DeltaTableConnector:
             SET s3_endpoint='{endpoint_URL}';
             SET s3_access_key_id='{access_key}';
             SET s3_secret_access_key='{password}';
+
+            -- HTTP reliability settings
+            SET http_keep_alive=false;
+            SET http_retries=6;
+            SET http_retry_wait_ms=1000;
+            SET http_timeout=30000;
+            SET enable_http_metadata_cache=false;
             """
         else:
-            auth_query = ""
+            auth_query = """
+            -- Set basic retry settings even for local files
+            SET http_retries=3;
+            SET http_retry_wait_ms=500;
+            SET http_timeout=15000;
+            """
 
         return auth_query
 
@@ -111,7 +149,8 @@ class DeltaTableConnector:
         if self.s3_credentials:
             storage_options = {
                 "AWS_ALLOW_HTTP": "true",
-                "AWS_S3_ALLOW_UNSAFE_RENAME": "true",
+                "AWS_S3_ALLOW_UNSAFE_RENAME": "false",
+                "AWS_CONDITIONAL_PUT": "etag",
             }
             storage_options.update(self.s3_credentials)
         else:
@@ -168,16 +207,16 @@ class DeltaTableConnector:
 
         query = auth_query + content_query
 
-        con = duckdb.connect()
-        con.execute(query)
-        data = con.fetchdf()
+        with duckdb.connect() as con:
+            con.execute(query)
+            data = con.fetchdf()
 
-        if not data.empty:
-            return self.interpreter.interpret(data, table)
-        else:
-            raise EmptyDeltaTableException(
-                f"block_ids {block_ids} not found in table {table}"
-            )
+            if not data.empty:
+                return self.interpreter.interpret(data, table)
+            else:
+                raise EmptyDeltaTableException(
+                    f"block_ids {block_ids} not found in table {table}"
+                )
 
     def __getitem__(self, kv: Tuple[str, List[int]]):
         table, key = kv
